#!/bin/sh
#
# This is a driver installer for UniChip controller.
#
# Version: 1.05.2407
# Released: 2009/12/07
#

version="1.00"
trash="/dev/null"
paramfile="/etc"
xmodulespath="/usr/X11R6/lib/modules/input"
xmodulespath2="/usr/lib/xorg/modules/input"
driver="unichip_drv.so "
driverpath="/unichip/module"
installpath="/"
identifier="TouchScreen"
#shortcutpath="/usr/bin"
xfilepath="/etc/X11/xorg.conf"
xfilepath2="/etc/X11/XF86Config-4"
xfilepath3="/etc/X11/XF86Config"
rclocalpath="/etc/rc.local"
xorgpath3="/usr/lib/X11/xorg.conf.d" # for Ubuntu 10.04 and Meego 1.0 netbook
xorgpath2="/usr/share/X11/xorg.conf.d" # for Fedora 13/14 & Ubuntu 10.10
xorgpath="/etc/X11/xorg.conf.d"
xorgfile="60-unichip.conf" # for Ubuntu 10.04 & Meego 1.0

ShowTitle() {
    echo ""
    echo "(*) Linux driver installer for UniChip controller "
    echo ""
}

CheckPermission() {
    echo -n "(I) Check user permission:"
    account=`whoami`
    if [ ${account} = "root" ]; then
        echo " ${account}, you are the supervisor."
    else
        echo " ${account}, you are not the supervisor."
        echo "(E) The root permission is required to run this installer."
        echo ""
        exit 1
    fi
}

CheckOldDaemon() {
	
    pidlist=`ps -ef|grep UcDriver| grep -v "grep"|awk '{print $2}'`
    if [ "$pidlist" = "" ]
    then
    {
	echo ""	
    }
    else
    for pid in ${pidlist}
    do
    kill -9 $pid

    done
    fi

    checkprocess="ps ax"
    processfile="process.info"
    ${checkprocess} > ${processfile} 2> ${trash}
    grep -q "UcDriver" ${processfile}
    if [ $? -eq 0 ]; then
        echo "(W) Found the old UniChip daemon: UcDriver."
       # echo "(E) It has to be removed before run this installer."
        echo ""
        rm -f ${processfile}
        exit 1
    fi
	rm -f ${processfile}
}

CheckXorgVersion() {
    checkx="X -version"
    xfile="X.info"
    ${checkx} > ${xfile} 2>&1
    grep -q "X.Org X Server 1.4." ${xfile}
    if [ $? -eq 0 ]; then
        grep -q "X.Org X Server 1.4.99" ${xfile}
        if [ $? -eq 0 ]; then
            echo "(I) Check X window version: 1.4.99.x" # FC9: 1.4.99.901
            xdirectory="x1499"
        else
            echo "(I) Check X window version: 1.4.x" # openSuSE 11 / Ubuntu 8.04: 1.4.0.90
            xdirectory="x14"
        fi
    else
        grep -q "X Window System Version 1.3." ${xfile} # FC8 / Ubuntu 7.10: 1.3.0
        if [ $? -eq 0 ]; then
            echo "(I) Check X window version: 1.3.x"
            xdirectory="x13"
        else
            grep -q "X.Org X Server 1.5." ${xfile} # Ubuntu 8.10
            if [ $? -eq 0 ]; then
                echo "(I) Check X window version: 1.5.x"
                xdirectory="x15"
            else
                grep -q "X.Org X Server 1.6." ${xfile} # Ubuntu 9.04
                if [ $? -eq 0 ]; then
                    echo "(I) Check X window version: 1.6.x"
                    xdirectory="x16"
                else
                    grep -q "X.Org X Server 1.7." ${xfile} # Fedora 12
                    if [ $? -eq 0 ]; then
                        echo "(I) Check X window version: 1.7.x"
                        xdirectory="x17"
                    else
                        grep -q "X.Org X Server 1.8." ${xfile} # Fedora 13
                        if [ $? -eq 0 ]; then
                            echo "(I) Check X window version: 1.8.x"
                            xdirectory="x18"
                        else
                            grep -q "X.Org X Server 1.9." ${xfile} # Fedora 14 & Ubuntu 10.10
                            if [ $? -eq 0 ]; then
                                echo "(I) Check X window version: 1.9.x"
                                xdirectory="x19"
                            else
                                grep -q "X.Org X Server 1.10." ${xfile} # Fedora 15 & Ubuntu 11.04
                                if [ $? -eq 0 ]; then
                                    echo "(I) Check X window version: 1.10.x"
                                    xdirectory="x110"
                                else
                                    grep -q "X Window System Version 6.8.99.900" ${xfile} # Mandriva 2006
                                    if [ $? -eq 0 ]; then
                                        echo "(I) Check X window version: 6.8.99.900"
                                        xmodule="egalax_drv.o"
                                        xdirectory="x67"
                                    else
                                        echo "(I) Check X window version: 6.9.0 ~ 7.2.0"
                                        xdirectory="x69"
                                    fi
                                fi
                            fi
                        fi
                    fi
                fi
            fi
        fi
    fi
    rm -f ${xfile}
}

CopyXorgMudule() {
    if [ -e ${xmodulespath}/mouse_drv.*o ]; then
        mousepath=`find ${xmodulespath} -name evdev_drv.*o`
    elif [ -e ${xmodulespath2}/mouse_drv.*o ]; then
        mousepath=`find ${xmodulespath2} -name evdev_drv.*o`
    else
        echo "(I) Searching the X input modules directory."
        mousepath=`find /usr -name evdev_drv.*o`
        if [ -z "${mousepath}" ]; then
            RemoveDriver
            echo "(E) No X input modules directory found."
            echo ""
            exit 1
        fi
    fi

    mouse=${mousepath##*/}
    if [ ${mouse} = "evdev_drv.so" ]; then
        xmodule=$driver
        CheckXorgVersion
        xpath=${mousepath%/*}
    elif [ ${mouse} = "evdev_drv.o" ]; then
        xmodule=$driver
        xdirectory="x67"
        xpath=${xmodulespath}
        echo "(I) Check X window version: 6.7 ~ 6.8.2"
    else
        RemoveDriver
        echo "(E) Unknown mouse module!"
        echo ""
        exit 1
    fi

    if [ -e ${xpath}/${xmodule} ]; then
        rm -f ${xpath}/${xmodule}
    fi
    echo "(I) Copy X module: ${xdirectory}/${xmodule} to ${xpath}."
    cp -f ${driverpath}/${xdirectory}/${xmodule} ${xpath}
}
RemoveXorgModule() {
    xmodule=$driver
    if [ -e ${xmodulespath}/${xmodule} ]; then
        rm -f ${xmodulespath}/${xmodule}
    elif [ -e ${xmodulespath2}/${xmodule} ]; then
        rm -f ${xmodulespath2}/${xmodule}
    else
        echo "(I) Searching the X module of touch device."
        fullmodulepath=`find /usr -name ${xmodule}`
        if [ -z "${fullmodulepath}" ]; then
            RemoveDriver
            echo "(E) No X module of touch device found."
            echo ""
            exit 1
        elif [ -e ${fullmodulepath} ]; then
            rm -f ${fullmodulepath}
        fi
    fi
    echo "(I) Removed X module."
}



AddrcLOCAL(){

#	grep -q "/unichip/driver/tpaneld -f" ${rclocalpath}
	
	grep -q "### unichip Configuration Beginning ###" $1
        if [ $? -eq 1 ]; then
            echo "(I) Add touch configuration into $1."
		filelines=`cat $1 | wc -l`
	sed -i ''${filelines}'a\### unichip Configuration Beginning ###\
/unichip/driver/usbwatch &\
/unichip/driver/UcDriver -f\
### unichip Configuration End ###' $1

        else
            echo "(I)touch configuration ........ ."
        fi


}



CheckInterface() {

	if [ -e  "/etc/tpaneld.conf" ];then
     		rm -f /etc/tpaneld.conf
   	fi

	touch /etc/tpaneld.conf
	echo "Port = /dev/unichip_usb ">>/etc/tpaneld.conf

	
	if [ -e /unichip ];then
		rm -r -f /unichip
	fi


	if [ -e unichip ];then
		echo " "
	else 
		echo "(I)please goto the path of  setup.sh ,then run setup.sh "
		exit 1
	fi 

	cp -r -f  unichip  /    
	
	chmod -R 777 /unichip
	chmod 7777 /unichip/driver/usbwatch
	chmod 7777 /unichip/driver/UcDriver
	chmod 7777 /unichip/tools/cal
	chmod 7777 /unichip/tools/UcCal
	/unichip/driver/usbwatch &
	/unichip/driver/UcDriver

	AddrcLOCAL ${rclocalpath}
     
}




RemoveDriver() {
	if [ -e  "/etc/tpaneld.conf" ];then
     		rm -f /etc/tpaneld.conf
   	fi

	if [ -e  "/etc/UCdp.conf" ];then
     		rm -f /etc/UCdp.conf
   	fi


	if [ -e  "/etc/cal.conf" ];then
     		rm -f /etc/cal.conf
   	fi

	if [ -e /unichip ];then
		rm -r -f /unichip
	fi
	sed -i '/### unichip Configuration Beginning ###/,/### Touch Configuration End ###/d' ${rclocalpath}
}


AddConfiguration() {
    echo "(I) Add touch configuration into $1."
    filelines=`cat $1 | wc -l`
    sed -i ''${filelines}'a\### Touch Configuration Beginning ###\
Section "InputDevice"\
        Identifier "'${identifier}'"\
        Driver "unichip"\
        Option "Device" "/dev/tkdat1"\
        Option "Parameters" "'${paramfile}'"\
		  Option "MaxX" "4095"\
		  Option "MinX" "0"\
		  Option "MaxY" "4095"\
		  Option "MinY" "0"\
        Option "ScreenNo" "0"\
EndSection\
### Touch Configuration End ###' $1

    grep -q "Section \"ServerLayout\"" $1
    if [ $? -eq 0 ]; then
        sed -i '/Section "ServerLayout"/a\        InputDevice "'${identifier}'" "SendCoreEvents"' $1
    else
        echo "(W) No \"ServerLayout\" section found! It will be appended automatically."
        sed -i '/### Touch Configuration Beginning ###/a\
Section "ServerLayout"\
        Identifier "Default Layout"\
        InputDevice "'${identifier}'" "SendCoreEvents"\
EndSection\
' $1
        screenfile="screen.info"
        sed '/Section "Screen"/,/EndSection/!d' $1 > ${screenfile} 2>&1
        sed -i '/Identifier/!d' ${screenfile} 2>&1
        screen=`sed 's/Identifier/Screen/g' ${screenfile}`
        if [ -n "${screen}" ]; then
            sed -i '/Identifier "Default Layout"/a\'"${screen}"'' $1
        fi
        rm -f ${screenfile}
    fi

    checkx="X -version"
    xfile="X.info"
    ${checkx} > ${xfile} 2>&1
    grep -q "X.Org X Server 1.4.0.90" ${xfile}
    if [ $? -eq 0 ]; then
        grep -q "Build Operating System: Linux Ubuntu" ${xfile} # Add AutoMapping except Ubuntu 8.04
        if [ $? -eq 1 ]; then
            sed -i '/Option "ScreenNo"/a\        Option "AutoMapping" "1"' $1
        fi
    fi

    usbfile="usb.info"
    checkusb="lsusb -v -d 0eef:0001"
    ${checkusb} > ${usbfile} 2> ${trash}
    grep -q "Current Operating System: Linux asus" ${xfile} # for EPC series
    if [ $? -eq 0 ]; then
        grep -q "Human Interface Device" ${usbfile}
        if [ $? -eq 0 ]; then
            sed -i '/Option "ScreenNo"/a\        Option "HidOnEPC" "1"' $1
        else
            grep -q "Vendor Specific Protocol" ${usbfile}
            if [ $? -eq 1 ]; then
                echo "(I) Note that the option \"HidOnEPC\" \"1\" should be appended in the touch"
                echo "    section of X configuration file for HID compliant touch controller."
            fi
        fi
    else
        grep -q "ubuntu" ${xfile} # for Ubuntu series
        if [ $? -eq 0 ]; then
            grep -q "Human Interface Device" ${usbfile}
            if [ $? -eq 0 ]; then
                sed -i '/Option "ScreenNo"/a\        Option "SkipClick" "1"' $1
            fi
        fi
    fi
    rm -f ${usbfile}
    rm -f ${xfile}
}

RemoveConfiguration() {
    sed -i '/"'${identifier}'" "SendCoreEvents"/d' $1
    sed -i '/### Touch Configuration Beginning ###/,/### Touch Configuration End ###/d' $1
    echo "(I) Removed touch configuration from $1."
}

EditConfigFile() {
     if [ -e ${xorgpath}/60-unichip.conf ]; then
		rm    ${xorgpath}/60-unichip.conf
    fi
    if  [ -e ${xorgpath2}/60-unichip.conf ]; then
		rm ${xorgpath2}/60-unichip.conf 
   fi 
    if [ -e ${xorgpath3}/60-unichip.conf ]; then
		rm  ${xorgpath3}/60-unichip.conf  
   fi

    if [ -w ${xfilepath} ]; then
        echo "(I) Found X configuration file: ${xfilepath}."
        grep -q "### Touch Configuration Beginning ###" ${xfilepath}
        if [ $? -eq 1 ]; then
            AddConfiguration ${xfilepath} ${device}
        else
            RemoveConfiguration ${xfilepath}
            AddConfiguration ${xfilepath} ${device}
        fi
    elif [ -w ${xfilepath2} ]; then
        echo "(I) Found X configuration file: ${xfilepath2}."
        grep -q "### Touch Configuration Beginning ###" ${xfilepath2}
        if [ $? -eq 1 ]; then
            AddConfiguration ${xfilepath2} ${device}
        else
            RemoveConfiguration ${xfilepath2}
            AddConfiguration ${xfilepath2} ${device}
        fi
    elif [ -w ${xfilepath3} ]; then
        echo "(I) Found X configuration file: ${xfilepath3}."
        grep -q "### Touch Configuration Beginning ###" ${xfilepath3}
        if [ $? -eq 1 ]; then
            AddConfiguration ${xfilepath3} ${device}
        else
            RemoveConfiguration ${xfilepath3}
            AddConfiguration ${xfilepath3} ${device}
        fi
    else

    	if [ -e ${xorgpath} ]; then
	
		cp  unichip/rule/60-unichip.conf  ${xorgpath}
	
	elif  [ -e ${xorgpath2} ]; then
		cp  unichip/rule/60-unichip.conf  ${xorgpath2}
	
	elif [ -e ${xorgpath3} ]; then
		cp  unichip/rule/60-unichip.conf  ${xorgpath3}
	else 
		#mkdir /etc/X11/xorg.conf.d
		#cp  unichip/rule/60-unichip.conf  ${xorgpath}
		Xorg :1 -configure
		sleep 5
		cp /root/xorg.conf.new /etc/X11/xorg.conf
		AddConfiguration ${xfilepath} ${device}
	fi
	
        exit 1
    fi
}

RestoreConfigFile() {
    if [ -w ${xfilepath} ]; then
        RemoveConfiguration ${xfilepath}
    elif [ -w ${xfilepath2} ]; then
        RemoveConfiguration ${xfilepath2}
    elif [ -w ${xfilepath3} ]; then
        RemoveConfiguration ${xfilepath3}
    fi

   	if [ -e ${xorgpath}/60-unichip.conf ]; then
		rm    ${xorgpath}/60-unichip.conf
    	 fi
    	 if  [ -e ${xorgpath2}/60-unichip.conf ]; then
		rm ${xorgpath2}/60-unichip.conf 
  	 fi 
   	 if [ -e ${xorgpath3}/60-unichip.conf ]; then
		rm  ${xorgpath3}/60-unichip.conf  
   	 fi
      echo "(E) Remove X configuration file ."
        echo ""
        exit 1
    
}

clear
ShowTitle

if [ $# = 0 ]; then
   CheckPermission
   CheckOldDaemon
   CheckInterface
   CopyXorgMudule
   EditConfigFile
    echo ""
    echo "(I) Please reboot the system or X Window for some changes to take effect."
    echo "(I) After booting, type /unichip/tools/UcCal to do calibration."
elif [ $# = 1 ]; then
   if [ $1 = "uninstall" ]; then
        CheckPermission
        echo "(I) Begin to remove UniChip driver."
        RemoveDriver
        RemoveXorgModule
        RestoreConfigFile
        echo ""
        echo "(I) The UniChip driver has been removed successfully."
        echo "(I) Please reboot the system for some changes to take effect."
    elif [ $1 = "version" ]; then
        echo "(I) Version: ${version}"
    else
        echo "(I) Usage: sh $0 or sh $0 uninstall"
    fi
else
    echo "(I) Usage: sh $0 or sh $0 uninstall"
fi

echo ""
